Struct CheckChangeTicks
pub struct CheckChangeTicks(/* private fields */);Expand description
An Event that can be used to maintain Ticks in custom data structures, enabling to make
use of bevy’s periodic checks that clamps ticks to a certain range, preventing overflows and thus
keeping methods like Tick::is_newer_than reliably return false for ticks that got too old.
Example
Here a schedule is stored in a custom resource. This way the systems in it would not have their change
ticks automatically updated via World::check_change_ticks,
possibly causing Tick-related bugs on long-running apps.
To fix that, add an observer for this event that calls the schedule’s
Schedule::check_change_ticks.
use bevy_ecs::prelude::*;
use bevy_ecs::change_detection::CheckChangeTicks;
#[derive(Resource)]
struct CustomSchedule(Schedule);
world.add_observer(|check: On<CheckChangeTicks>, mut schedule: ResMut<CustomSchedule>| {
schedule.0.check_change_ticks(*check);
});Implementations
impl CheckChangeTicks
impl CheckChangeTicks
pub fn present_tick(self) -> Tick
pub fn present_tick(self) -> Tick
Get the present Tick that other ticks get compared to.